!*************************************************************************
! COPYRIGHT (C) 1999 - 2002  EDF R&D
! THIS LIBRARY IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
! IT UNDER THE TERMS OF THE GNU LESSER GENERAL PUBLIC LICENSE 
! AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION; 
! EITHER VERSION 2.1 OF THE LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.
!
! THIS LIBRARY IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
! WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
! MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
! LESSER GENERAL PUBLIC LICENSE FOR MORE DETAILS.
!
! YOU SHOULD HAVE RECEIVED A COPY OF THE GNU LESSER GENERAL PUBLIC LICENSE
! ALONG WITH THIS LIBRARY; IF NOT, WRITE TO THE FREE SOFTWARE FOUNDATION,
! INC., 59 TEMPLE PLACE, SUITE 330, BOSTON, MA 02111-1307 USA
!
!**************************************************************************

! ******************************************************************************
! * - Nom du fichier : test9.f90
! *
! * - Description : lecture des familles d'un maillage MED 
! *                 via les routines de bas niveau
! *
! ******************************************************************************
program test9
  
  implicit none
  include 'med.hf'
  

! ******************************************************************************
! * Description :
! *    - ouverture du fichier
! *    - lecture des familles
! *    - fermeture du fichier
! ******************************************************************************

  integer        ret,cret,fid
  character*32   maa
  integer        mdim
  integer        nfam
  integer        i,j
  integer        natt,ngro
  character*200, allocatable, dimension (:) :: attdes    
  character*80,  allocatable, dimension (:) :: gro       
  integer,       allocatable, dimension (:) :: attval,attide
  character*32   nomfam
  integer        numfam


!  ** ouverture du fichier **
  call efouvr(fid,'test8.med',MED_LECT, cret)
  print *,cret

!  ** info sur le maillage **
  if (cret.eq.0) then
     call efmaai(fid,1,maa,mdim,cret)
     print *,"Maillage de nom : ",maa," et de dimension : ", mdim
  endif
  print *,cret

!  ** nombre de familles **
  if (cret .eq. 0) then
     call efnfam(fid,maa,0,0,nfam,cret)
     print *,' Nombre de familles a lire : ',nfam
  endif
  print *,cret

!  ** lecture de chaque famille **
  if (cret .eq. 0) then
     do i=1,nfam
        
!	** nombre de groupes **
        if (cret .eq. 0) then
           call efnfam(fid,maa,i,MED_GROUPE,ngro,cret)
        endif
        print *,cret

!	** nombre d'attributs **
        if (cret .eq. 0) then
           call efnfam(fid,maa,i,MED_ATTR,natt,cret)
        endif
        print *,cret

        print *,"Famille ",i," a ",natt," attributs et ",ngro," groupes " 

!	** nom,numero,attributs,groupes **
        if (cret .eq. 0) then
           allocate(attide(natt),attval(natt),attdes(natt),gro(ngro),STAT=ret)
!              print *,ret

           call effami(fid,maa,i,nomfam,numfam,attide,     &
                &                     attval,attdes,natt,gro,ngro,cret)
           print *,cret
           print *,"Famille de nom ",nomfam," et de numero ",numfam
           print *,"Attributs :"
           do j=1,natt
              print *,"ide = ",attide(j)," - val = ",attval(j)," - des = ",attdes(j)
           enddo
           deallocate(attide,attval,attdes)

           do j=1,ngro
              print *,"gro = ",gro(j)
           enddo
           deallocate(gro)
        endif
     enddo
  endif
     
     
!  ** fermeture du fichier                                           **
     call efferm (fid,cret)
     print *,cret
     
     
   end program test9


